<?php
//v 2.0.2 - Taxonomy Search.


global $prefs, $types, $fields;

$prefs = array("max_query_terms"=>"25", "max_word_length"=>"25", "min_doc_freq"=>"1", 
			   "min_term_freq"=>"1", "min_word_length"=>"3", "num_displayed"=>'5');
$types = array("idea");
$fields = array("title", "proposal", "body", "taxonomy");

/**
 * Things to be indexed. 
 */
function similar_ideas_luceneapi_document_alter($doc, $node, $module, $type) {
  
  if ('node' != $type) {
    return;
  }  
  $fulltree = variable_get('similar_ideas_fulltree', '1'); // = 1 for also indexing parent terms.
	
  foreach ($node->taxonomy as $tax){	
		if ($fulltree == 1) {
			$parents = taxonomy_get_parents_all($tax->tid); //Including parents in taxonomy-index.
			foreach ($parents as $parent){		
				$str_escape = str_replace(' ', '', $parent->name);
				$str_minus = strtolower ($str_escape);
				if ($str_minus != '') $taxonomy .= $str_minus." ";
			}		
		}else{
			$str_escape = str_replace(' ', '', $tax->name);
			$str_minus = strtolower ($str_escape);
			if ($str_minus != '') $taxonomy .= $str_minus." ";
		}
  }
  
  luceneapi_field_add($doc, 'unstored', 'taxonomy', $taxonomy);
  luceneapi_field_add($doc, 'unstored', 'title', $node->title);
  
  if (module_exists('ideastream')) luceneapi_field_add($doc, 'unstored', 'contents', $node->body);
  else luceneapi_field_add($doc, 'unstored', 'contents',  $node->field_id_proposal[0]["value"]);
}


function similar_ideas_luceneapi_queue_create(Zend_Search_Lucene_Interface $index, $text, $fields, $throw_exception = FALSE) {
	global $prefs;
	$priority_queue = array();
	if (!is_array($fields)) $fields = array();
	try {
		// checks if caching is enabled, attempts to retrieve cached results
		$results_cached = FALSE;
		if ($caching_enabled = luceneapi_variable_get('luceneapi_node', 'caching_enabled')) {
			$cid = sprintf('similar_ideas:%s', sha1((string)$text));
			if ($cache = cache_get($cid, LUCENEAPI_CACHE_TABLE)) {
				$priority_queue = (array)$cache->data;
				if (luceneapi_log_level_compare(WATCHDOG_DEBUG)) {
					$message = 'Priority queue cache hit for node %nid.';
					$variables = array('%nid' => $nid);
					watchdog('similar_ideas', $message, $variables, WATCHDOG_DEBUG);
				}
				$results_cached = TRUE;
			}
		}
		// if cache not hit, gets the priority queue
		if (!$results_cached) {
			// tokenizes the passed text into an array of term objects
			$token_list = Zend_Search_Lucene_Analysis_Analyzer::getDefault()->tokenize(
						  luceneapi_strip_tags($text), LUCENEAPI_ENCODING);
			// gets term frequencies for words in the text
			$tf = array();
			foreach ($token_list as $token) {
				$word = $token->getTermText();
				$strlen = drupal_strlen($word);
				if ($strlen < $prefs["min_word_length"] || ($prefs["max_word_length"] && $strlen > $prefs["max_word_length"])) {
					continue;
				}
				$tf[$word] = !isset($tf[$word]) ? 1 : $tf[$word] + 1;
			}
			// strips words with frequency below the limit
			if ($prefs["min_term_freq"]) {
				foreach ($tf as $word => $freq) {
					if ($freq < $prefs["min_term_freq"]) {
						unset($tf[$word]);
					}
				}
			}
			// gets total number of documents, opens similarity object
			$num_docs = luceneapi_index_size_get($index, FALSE, TRUE);
			$similarity = $index->getSimilarity();
			// calculates word scores
			foreach ($tf as $word => $term_freq) {
				// score based on inverse document frequency
				$doc_freq = 0;
				$top_field = 0;
				foreach ($fields as $field) {
					$freq = luceneapi_docfreq_get($index, $word, $field, TRUE);
					$top_field = ($freq > $doc_freq) ? $field : $top_field;
					$doc_freq = ($freq > $doc_freq) ? $freq : $doc_freq;
				}
				// continue to next term if we didn't find a match
				if (0 === $top_field) {
					continue;
				}
				$idf = $similarity->idfFreq($doc_freq, $num_docs);
				$score = $term_freq * $idf;
				// builds "res" object (@see Java Lucene source)
				$res = new stdClass();
				$res->word = $word;
				$res->top_field = $top_field;
				$res->score = $score;
				$res->doc_freq = $doc_freq;
				$res->tf = $term_freq;
				// appends object to priority queue
				$priority_queue[] = $res;
			}
			// sorts object by score, crops at $prefs["max_query_terms"]
			usort($priority_queue, 'similar_ideas_luceneapi_rsort');
			$priority_queue = array_slice($priority_queue, 0, $prefs["max_query_terms"]);
		}
		// caches data if caching is enabled
		if ($caching_enabled && !$results_cached) {
			cache_set($cid, $priority_queue, LUCENEAPI_CACHE_TABLE, time() + 604800);
		}
	} catch (Exception $e) {
		luceneapi_handle_error($e, $throw_exceptions);
	}
	return $priority_queue;
}

function similar_ideas_luceneapi_rsort($a, $b) {
	if ($a->score == $b->score) return 0;
	return ($a->score > $b->score) ? -1 : 1;
}

function similar_ideas_luceneapi_query_get(Zend_Search_Lucene_Interface $index, array $priority_queue) {
	try {
		if (!$query = luceneapi_query_get('boolean')) {
			throw new LuceneAPI_Exception(t('Error instantiating boolean query.'));
		}
		if (count($priority_queue)) {
			$first_item = TRUE;
			foreach ($priority_queue as $res) {
				if ($first_item) {
					$best_score = $res->score;
					$first_item = FALSE;
				}
				if (!$term_query = luceneapi_query_get('term', $res->word, $res->top_field)) {
					throw new LuceneAPI_Exception(t('Error instantiating term query.'));
				}
				$term_query->setBoost($res->score / $best_score);
				luceneapi_subquery_add($query, $term_query, 'neither', TRUE);
			}
		} else {
			if (!$empty = luceneapi_query_get('empty')) {
				throw new LuceneAPI_Exception(t('Error instantiating empty query.'));
			}
			luceneapi_subquery_add($query, $empty, 'required', TRUE);
		}
		return $query;
	} catch (LuceneAPI_Exception $e) {
		luceneapi_throw_error($e, WATCHDOG_ERROR, 'similar_ideas');
	}
	return FALSE;
}

function similar_ideas_luceneapi_query_alter($query, $module, $type) {
	global $nid, $types;
	
	// old lucene API code, left for future reference
	/*
	if (!$nid || 'node' != $type) return;
	try {
		if (!$multiterm = luceneapi_query_get('multiterm')) {
			throw new LuceneAPI_Exception(t('Error instantiating multiterm query.'));
		}
		$node_types = node_get_types('types');
		foreach($node_types as $type) {
			$type = $type->type;
			if (!in_array($type, $types)) $excluded_types[$type] = $type;
		}
		foreach ($excluded_types as $type) {
			if ($type) {
				$term = luceneapi_term_get($type, 'type', TRUE);
				luceneapi_term_add($multiterm, $term, 'neither', TRUE);
			}
		}
		if (count($multiterm->getTerms())) {
			$multiterm->setBoost(LUCENEAPI_IRRELEVANT);
			luceneapi_subquery_add($query, $multiterm, 'prohibited', TRUE);
		}
	} catch (Exception $e) {
		luceneapi_throw_error($e, WATCHDOG_ERROR, $module);
		if ($empty_query = luceneapi_query_get('empty')) {
			luceneapi_subquery_add($query, $empty_query, 'required');
		} else {
			luceneapi_throw_error(
				t('Error instantiating empty query.'), WATCHDOG_ERROR, $module
			);
		}
	} */
	
	//new code that adds excluded content types as separate queries (instead of multi term query)
	if (!$nid || 'node' != $type) return;
	try {
		if (!$multiterm = luceneapi_query_get('multiterm')) {
			throw new LuceneAPI_Exception(t('Error instantiating multiterm query.'));
		}
		$node_types = node_get_types('types');
		foreach($node_types as $type) {
			$type = $type->type;
			if (!in_array($type, $types)) $excluded_types[$type] = $type;
		}
		foreach ($excluded_types as $type) {
			if ($type) {
				//$term = luceneapi_term_get($type, 'type', TRUE);
				//luceneapi_term_add($multiterm, $term, 'neither', TRUE);
				
				$term_query = luceneapi_query_get('term', $type, 'type');
				luceneapi_subquery_add($query, $term_query, 'prohibited', TRUE);

			}
		}
		if (count($multiterm->getTerms())) {
			$multiterm->setBoost(LUCENEAPI_IRRELEVANT);
			//luceneapi_subquery_add($query, $multiterm, 'prohibited', TRUE);
		}
	} catch (Exception $e) {
		luceneapi_throw_error($e, WATCHDOG_ERROR, $module);
		if ($empty_query = luceneapi_query_get('empty')) {
			luceneapi_subquery_add($query, $empty_query, 'required');
		} else {
			luceneapi_throw_error(
				t('Error instantiating empty query.'), WATCHDOG_ERROR, $module
			);
		}
	}
}

function similar_ideas_luceneapi_text_get($nid, $tax_similarity, $distance, $vocabulary) {
	$text = '';
	$content = array();
	if ($node = node_load($nid)) {
		//The content we are searching similiraty with
		if (!$tax_similarity){
			$content[] = $node->title;
			if (module_exists('ideastream')) $content[] = $node->body;				
			else  $content[] = $node->field_id_proposal[0]["value"];				
		}
		else{
			foreach ($node->taxonomy as $tax){
				if ($tax->vid == $vocabulary || $vocabulary == 'all'){
					$str_escape = str_replace(' ', '', $tax->name);
					$str_minus = strtolower ($str_escape);
					if ($str_minus != '') $taxo.= $str_minus." ";
				}			
			}					
			
			$content[] = $taxo;
			
		}
		//var_dump($content); //Para ver que contenido se esta usando en cada tipo de search. 
		$text = join(' ', array_map('luceneapi_strip_tags', $content));
	}
	return $text;
}

function similar_ideas_luceneapi($node_id, $tax_similarity=FALSE, $distance=0, $vocabulary='all') {
	global $prefs, $fields;
	
	if (!$node_id) return '';
	else {
		global $nid;
		$nid = $node_id;
	}
	$items = array();
	try {
		if (!$index = luceneapi_index_open('luceneapi_node', $errstr)) {
			throw new LuceneAPI_Exception($errstr);
		}
		// gets the fields we will be searching
		$fields = drupal_map_assoc($fields);
		if (is_array($fields)) $fields = array_filter($fields);
		else $fields = array();
		// instantiates query container
		if (!$query = luceneapi_query_get('boolean')) {
			throw new LuceneAPI_Exception(t('Error instantiating boolean query.'));
		}
		// gets source text, builds the similar_ideas_luceneapi query
		$text = similar_ideas_luceneapi_text_get($nid,$tax_similarity, $distance, $vocabulary);
		$priority_queue = similar_ideas_luceneapi_queue_create($index, $text, $fields, TRUE);
		if ($mlt_query = similar_ideas_luceneapi_query_get($index, $priority_queue)) {
			luceneapi_subquery_add($query, $mlt_query, 'required', TRUE);
			// adds subquery to exclude current page
			$nid_query = luceneapi_query_get('term', $nid, 'nid');
			luceneapi_subquery_add($query, $nid_query, 'prohibited', TRUE);
			// allows modules to alter the query
			list($module, $type) = luceneapi_index_module_type_get($index);
			module_invoke_all('luceneapi_query_alter', $query, $module, $type);
			// executes query, pages number of results according to setting
			$hits = luceneapi_find($index, $query, array(), $positive_keys, TRUE);
		
			//CONFIG//				
			$length = variable_get('similar_ideas_ndisplayed', 5);
			$hits = array_slice($hits, 0, $length);
			// builds array of items passed to theme functions
			foreach ($hits as $hit) {
				try {
					// Attempts to load the node.
					if (!$node = node_load($index->getDocument($hit->id)->getFieldValue('nid'))) {
						throw new LuceneAPI_Exception(t('Error loading node.'));
					}
					$items[] = array(
						'title' => $node->title,
						'url' => url('node/'. $node->nid, array('absolute' => TRUE)),
						'node' => $node->nid,);
				} catch (Exception $e) {
					$items[] = array(
						'title' => t('Error loading page'),
						'url' => url($_GET['q']),
						'node' => 0,);
				}
			}
		}
	} catch (LuceneAPI_Exception $e) {
		luceneapi_handle_error($e, $throw_exceptions);
	}
	return $items;
}



